/*  job_computedifferentialequations.h
 *
 *  Copyright (C) 2010-2012 Andreas von Manteuffel
 *  Copyright (C) 2010-2012 Cedric Studerus
 *
 *  This file is part of the package Reduze 2.
 *  It is distributed under the GNU General Public License version 3
 *  (see the file GPL-3.0.txt or http://www.gnu.org/licenses/gpl-3.0.txt).
 */

#ifndef JOB_COMPUTEDIFFERENTIALEQUATIONS_H_
#define JOB_COMPUTEDIFFERENTIALEQUATIONS_H_

#include "job.h"
#include "int.h"
#include "ginac/ginac.h"

namespace Reduze {

/// Job to compute differential equations for master integrals
class ComputeDifferentialEquations: public Reduze::Job {
public:
	static YAMLSpec yaml_spec() {
		YAMLSpec s;
		s.set_keyword("compute_differential_equations");
		s.set_short_description(
				"Computes derivatives of integrals wrt invariants.");
		s.set_long_description(""//
					"Job to computes derivatives of integrals wrt invariants.");
		s.add_option("integrals_file", true, "string", ""//
					"Name of input file containing the integrals for which"
					" derivatives should be computed.");
		s.add_option("output_file", true, "string", "Output file name.");
		s.add_option("differentiation_invariants", true, "list of strings", ""//
					"Invariants w.r.t. which derivatives should be taken. The"
					" invariants must be expressions in terms of the globally"
					" defined kinematic invariants or scalar products of external"
					" momenta like SP(p1,p2), they are parsed by GiNaC.");
		s.add_option("auxiliary_invariants", false, "list of strings", ""//
					"Invariants which complete the differentation_invariants"
					" in the off-shell regime, but are kept fixed when calculating"
					" the requested derivatives.");
		s.add_options(Job::yaml_spec());
		return s;
	}
	virtual YAMLSpec yaml_spec_link() const {
		return yaml_spec();
	}
	ComputeDifferentialEquations() {
		add_auto_options();
	}
	virtual ~ComputeDifferentialEquations() {
	}
	virtual void run_serial();
	virtual bool find_dependencies(const std::set<std::string>&,//
			std::list<std::string>&, std::list<std::string>&, std::list<Job*>&);
	virtual std::string get_description() const;
protected:
	virtual void add_auto_options() {
		add_auto_io("integrals_file", integrals_filename_);
		add_auto_io("output_file", output_filename_);
	}
	virtual void print_manual_options(YAML::Emitter& os) const;
	virtual void read_manual_options(const YAML::Node&);
private:
	std::string integrals_filename_;
	std::string output_filename_;
	GiNaC::lst differentiation_invariants_, auxiliary_invariants_;
};

}

#endif /* JOB_COMPUTEDIFFERENTIALEQUATIONS_H_ */
