/*  job_printsectorinfo.h
 *
 *  Copyright (C) 2010-2012 Andreas von Manteuffel
 *  Copyright (C) 2010-2012 Cedric Studerus
 *
 *  This file is part of the package Reduze 2.
 *  It is distributed under the GNU General Public License version 3
 *  (see the file GPL-3.0.txt or http://www.gnu.org/licenses/gpl-3.0.txt).
 */

#ifndef JOB_PRINTSECTORINFO_H_
#define JOB_PRINTSECTORINFO_H_

#include "job.h"
#include "sector.h"
//#include "ginac/ginac.h"

namespace Reduze {

/// Job to print general information about a sector
class PrintSectorInfo: public Job {
public:
	static YAMLSpec yaml_spec() {
		YAMLSpec s;
		s.set_keyword("print_sector_info");
		s.set_short_description(
				"Prints diagrams and other information for sectors.");
		s.set_long_description(""//
					"Job to print information about sectors such as their"
					" subsectors and graphs.");
		s.add_option("sector_selection", true, "sector_selection", ""//
					"A selection of sectors for which information should"
					" be printed.");
		s.add_option("sector_tree_dot_file", false, "string", ""//
					"File name for the sector tree dot-file."
					" This file can be converted to a graphic with the \"dot\" command."
					" If left empty, no sector tree is generated.");
		s.add_option("feynman_diagram_dot_file", false, "string", ""//
					"File name for the Feynman diagram dot-file."
					" This file can be converted to a graphic with the \"dot\" command."
					" If left empty, no Feynman diagrams are generated.");
		s.add_option("minimize_graphs_by_twists", false, "boolean", ""//
					"This option only takes effect if the"
					" \"feynman_diagram_dot_file\" is not empty."
					" See the online help of the job \"setup_sector_mappings\""
					" for its documentation.");
		s.add_option("construct_minimal_graphs", false, "boolean", ""//
					"This option only takes effect if the"
					" \"feynman_diagram_dot_file\" is not empty."
					" See the online help of the job \"setup_sector_mappings\""
					" for its documentation.");
		s.add_options(Job::yaml_spec());
		return s;
	}
	virtual YAMLSpec yaml_spec_link() const {
		return yaml_spec();
	}
	PrintSectorInfo() :
		minimize_graphs_by_twists_(true), construct_minimal_graphs_(false) {
		add_auto_options();
	}
	virtual ~PrintSectorInfo() {
	}
	virtual void run_serial();
	virtual bool find_dependencies(const std::set<std::string>&,//
			std::list<std::string>&, std::list<std::string>&, std::list<Job*>&);
	virtual std::string get_description() const;

protected:
	virtual void add_auto_options() {
		add_auto_io("sector_tree_dot_file", sector_tree_dot_file_);
		add_auto_io("feynman_diagram_dot_file", feynman_diagram_dot_file_);
		add_auto_io("minimize_graphs_by_twists", minimize_graphs_by_twists_);
		add_auto_io("construct_minimal_graphs", construct_minimal_graphs_);
	}
	virtual void print_manual_options(YAML::Emitter& os) const;
	virtual void read_manual_options(const YAML::Node&);
	virtual void init();

private:
	SectorSelection sector_selection_;
	std::string sector_tree_dot_file_, feynman_diagram_dot_file_;
	bool minimize_graphs_by_twists_;
	bool construct_minimal_graphs_;
};

}

#endif /* JOB_PRINTSECTORINFO_H_ */
