/*  streamutils.cpp
 *
 *  Copyright (C) 2010-2012 Andreas von Manteuffel
 *  Copyright (C) 2010-2012 Cedric Studerus
 *
 *  This file is part of the package Reduze 2.
 *  It is distributed under the GNU General Public License version 3
 *  (see the file GPL-3.0.txt or http://www.gnu.org/licenses/gpl-3.0.txt).
 */

#include "streamutils.h"
#include <fstream>
#include <iostream>
#include <stdexcept>
#include "functions.h"
#include "int.h"

namespace Reduze {

using std::streampos;
using std::streamoff;
using std::streamsize;
using std::ios_base;
using std::runtime_error;
//using std::ostream;

// ICharArrayBuffer

ICharArrayBuffer::ICharArrayBuffer(const char* arr, size_t len) :
	begin(arr), end(arr + len), current(begin) {
	setg(0, 0, 0);
	setp(0, 0);
}

std::streampos ICharArrayBuffer::seekpos(std::streampos sp,
		ios_base::openmode which) {
	if ((which & ios_base::in) && !(which ^ ios_base::in)) {
		if (sp < 0 || sp > (end - begin)) {
			return traits_type::eof();
		} else {
			current = begin + sp;
			return sp;
		}
	} else { // support only for input pointer and nothing else
		return traits_type::eof();
	}
}

std::streampos ICharArrayBuffer::seekoff(std::streamoff off,
		ios_base::seekdir way, ios_base::openmode which) {
	switch (way) {
	case ios_base::beg:
		return seekpos(off, which);
	case ios_base::cur:
		return seekpos(current - begin + off, which);
	case ios_base::end:
		return seekpos(end - begin + off, which);
	default:
		return traits_type::eof();
	}
}

streamsize ICharArrayBuffer::showmanyc() {
	return (end - current);
}

ICharArrayBuffer::int_type ICharArrayBuffer::underflow() {
	if (current == end)
		return traits_type::eof();
	return *current;
}

ICharArrayBuffer::int_type ICharArrayBuffer::uflow() {
	if (current == end)
		return traits_type::eof();
	return *current++;
}

ICharArrayBuffer::int_type ICharArrayBuffer::pbackfail(
		ICharArrayBuffer::int_type c) {
	if (current == begin || (c != traits_type::eof() && c != current[-1]))
		return traits_type::eof();
	return *--current;
}

// NullBuffer

NullBuffer::NullBuffer() {
	setp(0, 0);
	setg(0, 0, 0);
}

NullBuffer::int_type NullBuffer::overflow(int_type c) {
	return c;
}

NullBuffer::int_type NullBuffer::underflow() {
	return traits_type::eof();
}

NullBuffer::int_type NullBuffer::sync() {
	return 0;
}

} // namespace Reduze
