/*  streamutils.h
 *
 *  Copyright (C) 2010-2012 Andreas von Manteuffel
 *  Copyright (C) 2010-2012 Cedric Studerus
 *
 *  This file is part of the package Reduze 2.
 *  It is distributed under the GNU General Public License version 3
 *  (see the file GPL-3.0.txt or http://www.gnu.org/licenses/gpl-3.0.txt).
 */

#ifndef STREAMUTILS_H_
#define STREAMUTILS_H_

#include <streambuf>
#include <fstream>
#include <stdexcept>
#include <stack>

namespace Reduze {

/// buffer for reading from a char array
class ICharArrayBuffer: public std::streambuf {
public:
	ICharArrayBuffer(const char* arr, size_t len);
private:
	// override protected virtual methods
	std::streampos seekpos(std::streampos sp, std::ios_base::openmode which =
			std::ios_base::in);
	std::streampos seekoff(std::streamoff off, std::ios_base::seekdir way,
			std::ios_base::openmode which = std::ios_base::in);
	std::streamsize showmanyc();
	int_type underflow();
	int_type uflow();
	int_type pbackfail(int_type c = traits_type::eof());
private:
	const char* begin;
	const char* end;
	const char* current;
};

/// null buffer
class NullBuffer: public std::streambuf {
public:
	NullBuffer();
private:
	// override protected virtual methods
	int_type overflow(int_type c);
	int_type underflow();
	int_type sync();
};

} // namespace Reduze

#endif /* STREAMUTILS_H_ */
